"""
This module have cubical complex. bitmap_complex module has BitmapComplex class
for a similar purpose. BitmapComplex depends on the dipha's IMAGE_DATA format
and uses dipha's implementation of cubical filtrations. This module has
CubicalComplex module to handle cubes and their boundaries.

The current impelementation of CubicalComplex is slow.
"""

import itertools

import numpy as np


class CubicalComplex(object):
    def __init__(self, array, periodic):
        self.array = array
        self.ndim = self.array.ndim
        self.shape = array.shape
        if np.any(periodic):
            raise(RuntimeError("Peridic cubical complex is not supported yet"))

    def cube(self, coord, non_deg):
        return self.Qube(coord, non_deg)

    def cube_info(self, cube):
        return cube.coord, cube.non_deg

    def embeded_dimension(self):
        return self.ndim

    def dimension(self, cube):
        return np.sum(cube.non_deg)

    def boundary(self, cube):
        boundary = []
        coord, non_deg = self.cube_info(cube)
        sign = -1
        for j in range(self.ndim):
            if non_deg[j]:
                boundary.append((sign, self._set_qn(cube, j, coord[j], False)))
                boundary.append((-sign, self._set_qn(cube, j, coord[j] + 1, False)))
                sign *= -1

        return boundary

    def boundary_nosign(self, cube):
        return []

    def value(self, cube):
        index = np.empty((self.ndim,), dtype=int)
        maxvalue = -np.inf

        def iter(j):
            nonlocal maxvalue
            if j == self.ndim:
                maxvalue = max(self.array[tuple(index)], maxvalue)
            else:
                crd, ndeg = self._get_qn(cube, j)
                if ndeg:
                    for i in range(2):
                        index[j] = crd + i
                        iter(j+1)
                else:
                    index[j] = crd
                    iter(j+1)

        iter(0)
        return maxvalue

    def _set_qn(self, cube, j, val, non_deg):
        new_cube = self.Qube(np.copy(cube.coord), np.copy(cube.non_deg))
        new_cube.coord[j] = val
        new_cube.non_deg[j] = non_deg
        return new_cube

    def _get_qn(self, cube, j):
        return cube.coord[j], cube.non_deg[j]

    def all_k_cubes(self, k):
        if k < 0 or k > self.ndim:
            return []

        non_deg_indices = itertools.combinations(range(self.ndim), k)
        non_deg_bits = []
        for index in non_deg_indices:
            bits = np.zeros((self.ndim, ), dtype=bool)
            bits[list(index)] = True
            non_deg_bits.append(bits)

        def valid_coord_non_deg(crd, ndeg):
            return np.all(crd + ndeg.astype(int) < self.shape)

        return (self.cube(crd, ndeg)
                for crd in itertools.product(*(range(k) for k in self.shape))
                for ndeg in non_deg_bits
                if valid_coord_non_deg(crd, ndeg))

    def k_cubes_between(self, k, lower, upper):
        """Return all k-cubes whose values are in the interval (lower, upper)

        Note that the interval does not have both end.
        """
        return [cube for cube in self.all_k_cubes(k)
                if lower < self.value(cube) < upper]

    class Qube(object):
        def __init__(self, coord, non_deg):
            self.coord = coord
            self.non_deg = non_deg

        def __eq__(self, other):
            return (np.array_equal(self.coord, other.coord)
                    and np.array_equal(self.non_deg, other.non_deg))

        def __lt__(self, other):
            return ((tuple(self.coord), tuple(self.non_deg))
                    < (tuple(other.coord), tuple(other.non_deg)))

        def __repr__(self):
            return "Qube({}, {})".format(self.coord, self.non_deg)
