# You should run this script in the same directory of this file.

import os
import platform
import re

from setuptools import setup, Extension

import numpy as np

with open(os.path.join(os.path.dirname(__file__), "homcloud", "base.py")) as f:
    for line in f:
        m = re.match(r"__version__ = \"([\d.]+)\"", line)
        if m:
            __version__ = m.group(1)

LIBS = ["CGAL", "mpfr", "gmp"]

if platform.system() == 'Windows':
    LIBS = ["CGAL-vc140-mt-4.9", "libmpfr-4", "libgmp-10"]

EXT_MODULES = [
    Extension("homcloud.alpha_shape3",
              include_dirs=[np.get_include()],
              libraries=LIBS,
              extra_compile_args=["-std=c++11"],
              sources=["ext/alpha_shape3.cc"]),
    Extension("homcloud.alpha_shape2",
              include_dirs=[np.get_include()],
              libraries=LIBS,
              extra_compile_args=["-std=c++11"],
              sources=["ext/alpha_shape2.cc"]),
    Extension("homcloud.pict_tree",
              include_dirs=[np.get_include()],
              extra_compile_args=["-std=c++11"],
              sources=["ext/pict_tree.cc"]),
]

if os.environ.get("HOMCLOUD_BUILD_DIPHA_ALT", "0") == "1":
    EXT_MODULES.append(
        Extension("homcloud.dipha_alt_ext",
                  extra_compile_args=['-std=c++11'],
                  sources=[
                      "ext/dipha_alt/bitarray.cc",
                      "ext/dipha_alt/boundary_matrix.cc",
                      "ext/dipha_alt/dipha_alt.cc",
                      "ext/dipha_alt/filtration.cc",
                      "ext/dipha_alt/read_filtration.cc",
                      "ext/dipha_alt/write_diagram.cc",
                  ])
    )

# The following branch is requied since pytest-runner is not
# packaged on debian jessie.
# This will be removed when debian stretch is released.
if os.environ.get("HOMCLOUD_DISABLE_PYTEST_RUNNER", "0") == "1":
    SETUP_REQUIRES = []
else:
    SETUP_REQUIRES = ["pytest-runner"]

setup(
    name="homcloud-base",
    version=__version__,
    description="homcloud base tools",
    author="Ippei Obayashi",
    author_email="ippei.obayashi.d8@tohoku.ac.jp",
    url="http://www.wpi-aimr.tohoku.ac.jp/hiraoka_labo/homcloud",
    ext_modules=EXT_MODULES,
    packages=[
        "homcloud",
        "homcloud.pict",
    ],
    entry_points={
        'console_scripts': [
            "homcloud-dipha = homcloud.dipha:main",
            "homcloud-pc2diphacomplex = homcloud.pc2diphacomplex:main",
            "homcloud-pict-binarize = homcloud.pict.binarize:main",
            "homcloud-pict-binarize3d = homcloud.pict.binarize3d:main",
            "homcloud-pict-binarize-nd = homcloud.pict.binarize_nd:main",
            "homcloud-pict-pixel-levelset = homcloud.pict.pixel_levelset:main",
            "homcloud-pict-pixel-levelset-nd = homcloud.pict.pixel_levelset_nd:main",
            "homcloud-pict-merge-tree-levelset = homcloud.pict.merge_tree_levelset:main",
            "homcloud-diagram-to-text = homcloud.diagram_to_text:main",
            "homcloud-plot-diagram = homcloud.plot_diagram:main",
            "homcloud-pict-tree = homcloud.pict.tree:main",
        ],
    },
    install_requires=[
        "numpy",
        "matplotlib",
        "scipy",
        "scikit-learn",
        "msgpack-python",
        "Pillow",
    ],
    setup_requires=SETUP_REQUIRES,
    tests_require=[
        "pytest>=3.0",
        "pytest-mock",
    ],
    classifiers=[
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Intended Audience :: Science/Research",
    ],
)
